package com.hexin.android.view

import android.view.animation.AnimationUtils

/**
 * 循环跑马灯的帮助类，具体使用方法可参照FenshiStockInfo中的使用
 * */
class MarqueeHelper {
    companion object {
        private const val STATUS_STOP = 0 // 停止状态
        private const val STATUS_RUN  = 1 // 滚动状态
        private const val STATUS_COLD = 2 // 冷却状态（当一次滚动完成时暂听滚动的状态）
        const val COLD_TIME   = 1500L // 冷却时间
        const val MIN_ANIM_DURATION = 300
    }
    private var status = STATUS_STOP // 跑马灯状态
    private var startTime = 0L // 开始时间
    private var duration = MIN_ANIM_DURATION // 单圈跑动时间
    private var scrollWidth = 0f // 可滑动距离

    /**
     * 1
     * onDraw()中去用，第一次绘制结束再调用
     * 调用时先构造对象，再判断有没有初始化，没有初始化就老老实实初始化
     * @return 是否有初始化过
     */
    fun isInit(): Boolean {
        return scrollWidth > 0
    }

    /**
     * 2
     * 判断了上面没有初始化后，先初始化一下，这一步很重要，不要传一些蛇皮参数，会被拒绝初始化的
     * @param scrollWidth 跑马灯总共需要滑动的距离，必须为正数，不允许小于1
     * @param vel 速度时间单位为：pixel/s，不允许小于1
     */
    fun initMarquee(scrollWidth: Float, vel: Float) {
        if (scrollWidth <= 0 || vel <= 0 ) {
            throw IllegalArgumentException("scrollWidth($scrollWidth) and vel($vel) must > 0")
        }
        this.scrollWidth = scrollWidth
        duration = ((scrollWidth / vel) * 1000).toInt() // 直接把速度转换成时间间隔方便处理
        if (duration < MIN_ANIM_DURATION) { // 防止可滑动距离太短，滚动太快
            duration = MIN_ANIM_DURATION
        }
    }

    /**
     * 3
     * 初始化完成之后，紧接着调用这个方法
     */
    fun start(): Boolean {
        return if (isInit() && status == STATUS_STOP) {
            status = STATUS_RUN
            startTime = AnimationUtils.currentAnimationTimeMillis()
            true
        } else {
            false
        }
    }

    /**
     * 4
     * onDraw时判断，如果处于滚动状态，取偏移值
     * @return 是否正在滚动
     */
    fun isMarquee(): Boolean {
        return status != STATUS_STOP
    }

    /**
     * 5
     * onDraw时判断，如果处于静止状态，则调用postInvalidateDelayed(COLD_TIME)
     * @return 是否正在滚动
     */
    fun isCold(): Boolean {
        return status == STATUS_COLD
    }

    /**
     * 6
     * 处于滚动状态时，再调这个方法 canvas.translate(getDelta(), dy)
     * @return 偏移值
     */
    fun getDelta(): Float {
        return when(status) {
            STATUS_STOP -> 0f
            STATUS_COLD -> {
                val now = AnimationUtils.currentAnimationTimeMillis()
                if (now - startTime >= duration + COLD_TIME) {
                    status = STATUS_RUN
                    startTime = now
                }
                0f
            }
            STATUS_RUN -> {
                val fraction = (AnimationUtils.currentAnimationTimeMillis() - startTime) / duration.toFloat()
                var delta = fraction * scrollWidth
                if (delta >= scrollWidth) {
                    delta = scrollWidth
                    status = STATUS_COLD
                }
                -delta
            }
            else -> 0f
        }
    }

    fun cancel() {
        status = STATUS_STOP
        duration = 0
        scrollWidth = 0f
    }
}